export class Details extends HTMLElement {
  constructor() {
    super();
    this.attachShadow({ mode: "open" });
    this.name = "name";
    this.image =
      "https://static.showit.co/800/Xii7h1p6TuqyWFdYgzD8SA/shared/placeimagehere_p_1.jpg";
    this.buttonlabel = "buttonlabel";
    this.buttonlink = "buttonlink";
    this.description = "description";
  }

  render(html) {
    this.shadowRoot.innerHTML = "";
    this.shadowRoot.innerHTML = `${html}`;
  }

  connectedCallback() {
    const parentContainer = this.closest(".container");
    parentContainer.style.cssText =
      "width: 100%; margin: 0; padding: 0; max-width: 100%;";
    this.removeAttribute("hidden");
    this.render(`<style> 
    body {
      padding: 20px;
    }
    
    .container {
      display: flex;
      border: 1px solid #eaecef;
      height: 200px;
      padding: 1%;
      background-color: white;
      box-shadow: 2px 5px 5px 1px lightgrey;
    }
    
    .img-container {
      width: 15%;
      padding: 20px;
    }
    
    .img {
      border: 1px solid white;
      width: 100%;
      height: 100%;
      background-color: #babbbc;
    }
    
    .content {
      border: 1px solid white;
      flex-grow: 1;
      display: flex;
      flex-direction: column;
      padding: 20px;
      justify-content: space-between;
    }
    
    .stripe {
      border: 1px solid white;
      height: 20%;
      background-color: #babbbc;
    }
    
    .small-stripe {
      width: 40%;
    }
    
    .medium-stripe {
      width: 70%;
    }
    
    .long-stripe {
      width: 100%;
    }
    
    .container.loading .img, .container.loading .stripe {
      animation: hintloading 2s ease-in-out 0s infinite reverse;
      -webkit-animation: hintloading 2s ease-in-out 0s infinite reverse;
    }
    
    @keyframes hintloading
    {
      0% {
        opacity: 0.5;
      }
      50%  {
        opacity: 1;
      }
      100% {
        opacity: 0.5;
      }
    }
    
    @-webkit-keyframes hintloading
    {
      0% {
        opacity: 0.5;
      }
      50%  {
        opacity: 1;
      }
      100% {
        opacity: 0.5;
      }
    }
    </style>

    <div class='container loading'>
  <div class='img-container'>
    <div class='img'>
    </div>
  </div>
  <div class='content'>
    <div class='stripe small-stripe'>
    </div>
    <div class='stripe medium-stripe'>
    </div>
    <div class='stripe long-stripe'>
    </div>
  </div>
</div>`);
    const contentDiv = document.querySelector(".content");
    const softrAppId = contentDiv.getAttribute("data-appid");
    const recordId = window.location.href.split("=").pop();
    const apiURL = `https://xtsq-tb6s-h4xj.f2.xano.io/api:V1kisAip/list-details-3`;
    const attributeNodes = this.attributes;
    const attributeObject = {};

    // Iterate over the nodes in the NamedNodeMap
    for (let i = 0; i < attributeNodes.length; i++) {
      const node = attributeNodes[i];
      attributeObject[node.nodeName] = node.nodeValue;
    }

    //Add other info required to be passed to Xano into the "attributeObject" as this object will be converted to the "component" JSON input
    attributeObject.recordId = recordId;
    attributeObject.softrAppId = softrAppId;

    // Convert the attribute object to a string

    const attributeString = JSON.stringify(attributeObject);
    const component = `{"component":${attributeString}}`;

    fetch(apiURL, {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
      },
      body: component,
    })
      .then((response) => response.json())
      .then((data) => {
        this.render(data);
      });
  }

  static get observedAttributes() {
    return ["name", "image", "buttonlink", "buttonlabel", "description"];
  }

  attributeChangedCallback(name, oldValue, newValue) {
    const recordId = window.location.href.split("=").pop();

    /////HERE IS WHERE I AM 10th Dec 2022
    const xanoCallback = "Xano API Call Goes Here";

    this.render();
  }

  set name(val) {
    if (!this.getAttribute("name")) {
      this.setAttribute("name", val);
    }
  }

  get name() {
    return this.getAttribute("name");
  }

  set image(val) {
    if (!this.getAttribute("image")) {
      this.setAttribute("image", val);
    }
  }

  get image() {
    return this.getAttribute("image");
  }

  set buttonlabel(val) {
    if (!this.getAttribute("buttonlabel")) {
      this.setAttribute("buttonlabel", val);
    }
  }

  get buttonlabel() {
    return this.getAttribute("buttonlabel");
  }

  set buttonlink(val) {
    if (!this.getAttribute("buttonlink")) {
      this.setAttribute("buttonlink", val);
    }
  }

  get buttonlink() {
    return this.getAttribute("buttonlink");
  }

  set description(val) {
    if (!this.getAttribute("description")) {
      this.setAttribute("description", val);
    }
  }

  get description() {
    return this.getAttribute("description");
  }
}
