export class Layout extends HTMLElement {
  layouts = {
    v1: `<style>
    .responsive {
      container-type: inline-size;
      margin: 3%;
    }
  
    .card-container {
      width: 100%;
      margin: 0;
      max-width: 100%;
      display: grid;
      grid-template-columns: repeat(1, 1fr);
    }
  
    .header,
    .content,
    .footer {
      height: 96%;
      max-height: 100vh;
      overflow-y: auto;
      border: 1px solid #ccc;
      display: flex;
      flex-direction: column;
      text-align: center;
      padding-bottom: 5%;
      -ms-overflow-style: none; /* IE and Edge */
      scrollbar-width: none; /* Firefox */
    }

    ::slotted(*) {
        max-width: 100%;
        margin-bottom: 3%;
        margin-top: 0;
      }
  
    @container (max-width: 575px) {
      .card-container {
        grid-template-columns: repeat(1, 1fr);
      }

      .header,
    .content,
    .footer {
      height: 100%;
      max-height: 100%;
      padding-bottom: 0;
    }

    ::slotted(*) {
        margin-bottom: 0;
      }
    }
  
    @container (min-width: 576px) and (max-width: 767px) {
        .card-container {
            grid-template-columns: repeat(1, 1fr);
          }
    
          .header,
        .content,
        .footer {
          height: 100%;
          max-height: 100%;
          padding-bottom: 0;
        }
    
        ::slotted(*) {
            margin-bottom: 0;
          }
    }
  
    .header::-webkit-scrollbar,
    .content::-webkit-scrollbar,
    .footer::-webkit-scrollbar {
      display: none;
    }
  </style>
  <section class="responsive">
  <div class="card-container">
    <div class="header">
      <div class="header-content">
        <slot name="header"></slot>
      </div>
    </div>
    <slot name="cell">
      <div class="cell">
        <div class="cell-content">
          <slot name="cell-content"></slot>
        </div>
      </div>
    </slot>
  </div>
</section>`,
    v2: `<style>
.responsive {
  container-type: inline-size;
  margin: 3%;
}

.card-container {
  width: 100%;
  margin: 0;
  max-width: 100%;
  display: grid;
  grid-template-columns: repeat(2, 1fr);
}

.header,
.content,
.footer {
  height: 96%;
  max-height: 100vh;
  overflow-y: auto;
  border: 1px solid #ccc;
  display: flex;
  flex-direction: column;
  text-align: center;
  padding-bottom: 5%;
  -ms-overflow-style: none; /* IE and Edge */
  scrollbar-width: none; /* Firefox */
}

::slotted(*) {
    max-width: 100%;
    margin-bottom: 3%;
    margin-top: 0;
  }

@container (max-width: 575px) {
  .card-container {
    grid-template-columns: repeat(2, 1fr);
  }

  .header,
.content,
.footer {
  height: 100%;
  max-height: 100%;
  padding-bottom: 0;
}

::slotted(*) {
    margin-bottom: 0;
  }
}

@container (min-width: 576px) and (max-width: 767px) {
    .card-container {
        grid-template-columns: repeat(2, 1fr);
      }

      .header,
    .content,
    .footer {
      height: 100%;
      max-height: 100%;
      padding-bottom: 0;
    }

    ::slotted(*) {
        margin-bottom: 0;
      }
}

.header::-webkit-scrollbar,
.content::-webkit-scrollbar,
.footer::-webkit-scrollbar {
  display: none;
}
</style>
<section class="responsive">
<div class="card-container">
<div class="header">
  <div class="header-content">
    <slot name="header"></slot>
  </div>
</div>
<div class="content">
  <div class="content-content">
    <slot name="content"></slot>
  </div>
</div>
<slot name="cell">
  <div class="cell">
    <div class="cell-content">
      <slot name="cell-content"></slot>
    </div>
  </div>
</slot>
</div>
</section>`,
    v3: `<style>
  .responsive {
    container-type: inline-size;
    margin: 3%;
  }

  .card-container {
    width: 100%;
    margin: 0;
    max-width: 100%;
    display: grid;
    grid-template-columns: repeat(3, 1fr);
  }

  .header,
  .content,
  .footer {
    height: 96%;
    max-height: 100vh;
    overflow-y: auto;
    border: 1px solid #ccc;
    display: flex;
    flex-direction: column;
    text-align: center;
    padding-bottom: 5%;
    -ms-overflow-style: none; /* IE and Edge */
    scrollbar-width: none; /* Firefox */
  }

  ::slotted(*) {
      max-width: 100%;
      margin-bottom: 3%;
      margin-top: 0;
    }

  @container (max-width: 575px) {
    .card-container {
      grid-template-columns: repeat(3, 1fr);
    }

    .header,
  .content,
  .footer {
    height: 100%;
    max-height: 100%;
    padding-bottom: 0;
  }

  ::slotted(*) {
      margin-bottom: 0;
    }
  }

  @container (min-width: 576px) and (max-width: 767px) {
      .card-container {
          grid-template-columns: repeat(3, 1fr);
        }

        .header,
      .content,
      .footer {
        height: 100%;
        max-height: 100%;
        padding-bottom: 0;
      }

      ::slotted(*) {
          margin-bottom: 0;
        }
  }

  .header::-webkit-scrollbar,
  .content::-webkit-scrollbar,
  .footer::-webkit-scrollbar {
    display: none;
  }
  </style>
<section class="responsive">
<div class="card-container">
<div class="header">
  <div class="header-content">
    <slot name="header"></slot>
  </div>
</div>
<div class="content">
  <div class="content-content">
    <slot name="content"></slot>
  </div>
</div>
<div class="footer">
  <div class="footer-content">
    <slot name="footer"></slot>
  </div>
</div>
<slot name="cell">
  <div class="cell">
    <div class="cell-content">
      <slot name="cell-content"></slot>
    </div>
  </div>
</slot>
</div>
</section>`,
  };

  get type() {
    return this.getAttribute("type");
  }

  variant = ``;
  setVariant(type) {
    this.variant = this.layouts[type];
  }

  constructor() {
    super();

    const shadowRoot = this.attachShadow({ mode: "open" });
    const template = document.createElement("template");
    this.setVariant(this.type);
    if (this.variant) {
      template.innerHTML = `${this.variant}`;
      shadowRoot.appendChild(template.content.cloneNode(true));
    }

    if (!this.variant) {
      template.innerHTML = `${this.layouts.v1}`;
      shadowRoot.appendChild(template.content.cloneNode(true));
    }
  }

  render(html) {
    this.innerHTML = ``;
    this.innerHTML = `${html}`;
  }

  connectedCallback() {
    console.log("Connected!");
    console.log(this.innerHTML);
    const childNodes = this.childNodes;
    const htmlArray = [];

    // Iterate over the nodes in the NamedNodeMap
    for (let i = 0; i < childNodes.length; i++) {
      const node = childNodes[i];
      if (node.nodeName === "#text") continue;
      const attributesArray = [];
      [...node.attributes].forEach((attr) => {
        const nodeName = attr.nodeName;
        const nodeValue = attr.nodeValue;
        attributesArray.push({ [nodeName]: nodeValue });
      });

      const slot = attributesArray.find((el) =>
        el.hasOwnProperty("slot")
      )?.slot;
      const field = attributesArray.find((el) =>
        el.hasOwnProperty("field")
      )?.field;
      const src = attributesArray.find((el) => el.hasOwnProperty("src"))?.src;
      const nodeClass = attributesArray.find((el) =>
        el.hasOwnProperty("class")
      )?.class;
      const nodeId = attributesArray.find((el) => el.hasOwnProperty("id"))?.id;
      const href = attributesArray.find((el) =>
        el.hasOwnProperty("href")
      )?.href;

      let htmlString = "";

      if (field) {
        if (node.localName === "img") {
          htmlString = `<${node.localName} slot="${slot}" field="${field}" src="{{${field}}}">`;
        } else if (node.localName === "input") {
          htmlString = "";
        } else {
          htmlString = `<${node.localName} slot="${slot}" field="${field}">{{${field}}}</${node.localName}>`;
        }
      }

      if (!field) {
        if (node.localName === "img") {
          htmlString = `<${node.localName} slot="${slot}" src="${src}">`;
        } else if (node.localName === "input") {
          htmlString = "";
        } else {
          htmlString = `<${node.localName} slot="${slot}">${node.innerHTML}</${node.localName}>`;
        }
      }

      htmlArray.push({
        tag: node.localName,
        value: node.nodeValue,
        innerText: node.innerText,
        parentElement: node.parentElement.tagName,
        innerHTML: node.innerHTML,
        attributes: attributesArray,
        field: field,
        slot: slot,
        src: src,
        class: nodeClass,
        id: nodeId,
        href: href,
        htmlString: htmlString,
      });
    }
    console.log(htmlArray);

    const componentAttributes = [];
    [...this.attributes].forEach((attr) => {
      componentAttributes.push({ [attr.nodeName]: attr.nodeValue });
    });
    const base = componentAttributes.find((el) =>
      el.hasOwnProperty("base")
    )?.base;
    const table = componentAttributes.find((el) =>
      el.hasOwnProperty("table")
    )?.table;
    const recordId = componentAttributes.find((el) =>
      el.hasOwnProperty("record")
    )?.record;

    const softrAppId = "c55f4c03-a4a0-47b8-aa16-8526a857fcf8";
    htmlArray.unshift({
      componentName: this.tagName,
      componentAttributes: componentAttributes,
      softrAppId: softrAppId,
    });

    let elementsString = "";

    htmlArray.forEach((el) => {
      if (el.htmlString) {
        elementsString
          ? (elementsString = elementsString + ", " + el.htmlString)
          : (elementsString = el.htmlString);
      }
    });

    const elements = elementsString.split(", ");

    let fieldsString = "";

    htmlArray.forEach((el) => {
      if (el.componentName) return;
      const fieldValue = el.field ? el.field : "none";

      fieldsString
        ? (fieldsString = fieldsString + ", " + fieldValue)
        : (fieldsString = fieldValue);
    });

    const fields = fieldsString.split(", ");

    let tagsString = "";

    htmlArray.forEach((el) => {
      tagsString
        ? (tagsString = tagsString + ", " + el.tag)
        : (tagsString = el.tag);
    });

    const tags = tagsString.split(", ");

    const string = JSON.stringify(this.innerHTML);
    console.log(string);

    function findUniqueSubstrings(string, substringFormat) {
      const substrings = [];
      let startIndex = 0;
      while (startIndex < string.length) {
        const start = string.indexOf(substringFormat, startIndex);
        if (start === -1) {
          break;
        }
        let end = start + substringFormat.length;
        while (string[end] !== "}") {
          end++;
        }
        end++; // include the closing curly brace
        end++; // include the closing curly brace
        substrings.push(string.substring(start, end));
        startIndex = end;
      }
      return [...new Set(substrings)];
    }

    const placeholders = findUniqueSubstrings(string, "{{");

    const payload = {
      softrAppId: softrAppId,
      base: base,
      table: table,
      recordId: recordId,
      elements: elements,
      fields: fields,
      tags: tags,
      innerHTML: this.innerHTML,
      placeholders: placeholders,
    };
    console.log(payload);
    this.render(``);

    const payloadString = JSON.stringify(payload);
    const component = `{"component":${payloadString}}`;
    console.log(component);
    const apiURL = `https://xejw-c6td-wkd1.f2.xano.io/api:-D49NGOp/no-code-render`;

    fetch(apiURL, {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
      },
      body: component,
    })
      .then((response) => response.json())
      .then((data) => {
        this.render(data);
      });
  }
}
