export class List extends HTMLElement {
    connectedCallback() {
      this.innerHTML = `<p class="loading">Getting information...</p>`;
      // ... rest of the code
    }
  
    render(html) {
      this.querySelector(".loading")?.remove();
      const mode = this.mode === "save" ? "save" : "rendered";
      const element = document.createElement("no-code-details");
      element.setAttribute("source", this.source);
      element.setAttribute("api", this.api);
      element.setAttribute("base", this.base);
      element.setAttribute("table", this.table);
      element.setAttribute("record", "");
      element.setAttribute("type", this.type);
      element.setAttribute("mode", mode);
      element.innerHTML = html;
      if (this.mode === "save") {
        this.insertAdjacentElement("afterbegin", element);
      } else {
        this.appendChild(element);
      }
    }
  
    setVariant(type) {
      const apiURL = `https://xejw-c6td-wkd1.f2.xano.io/api:-D49NGOp/templates/${type}`;
  
      return fetch(apiURL, {
        method: "GET",
        headers: {
          "Content-Type": "application/json",
        },
      })
        .then((response) => response.json())
        .then((data) => {
          const variant = data.template.replaceAll("\u00A0", "");
          window.templateHTML[type] = variant;
        });
    }
  
    getData(config) {
      const apiURL = `https://xejw-c6td-wkd1.f2.xano.io/api:-D49NGOp/No-Code-List`;
  
      const payload = config;
      const payloadString = JSON.stringify(config);
      const component = `{"component":${payloadString}}`;
  
      return fetch(apiURL, {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: component,
      })
        .then((response) => response.json())
        .then((data) => {
          let records = [];
  
          if (data.records) {
            records = data.records;
          }
  
          this.dataFetched = [...this.dataFetched, ...records];
  
          let output = "";
  
          records.forEach((record) => {
            const template = window.templateHTML[this.type];
  
            let renderedTemplate = template;
  
            // Mapping the record data to template fields
            const fields = record.fields;
            for (const key in fields) {
              if (fields.hasOwnProperty(key)) {
                const regex = new RegExp(`{{\\s*${key}\\s*}}`, "g");
                renderedTemplate = renderedTemplate.replace(regex, fields[key]);
              }
            }
  
            output += renderedTemplate;
          });
  
          this.render(output);
        });
    }
  
    getSearchData(config) {
      const searchURL = `https://xejw-c6td-wkd1.f2.xano.io/api:-D49NGOp/No-Code-List/Retrieve`;
  
      const payload = config;
      const payloadString = JSON.stringify(payload);
      const component = `{"component":${payloadString}}`;
  
      return fetch(searchURL, {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: component,
      })
        .then((response) => response.json())
        .then((data) => {
          this.searchFetched = [...this.searchFetched, ...data.records];
          this.setFilterData(this.searchFetched);
        });
    }
  
    getFilterData(config) {
      const searchURL = `https://xejw-c6td-wkd1.f2.xano.io/api:-D49NGOp/No-Code-List/Filter`;
  
      const payload = config;
      const payloadString = JSON.stringify(payload);
      const component = `{"component":${payloadString}}`;
  
      return fetch(searchURL, {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: component,
      })
        .then((response) => response.json())
        .then((data) => {
          const filterOptions = data.field.options.choices;
          this.setFilterData(filterOptions);
        });
    }
  
    setFilterData(data) {
      const dropdownMenu = this.shadowRoot.querySelector(".dropdown-menu");
  
      const dropdownMenuItemTemplate = document.createElement("template");
  
      data.forEach((option) => {
        const optionText = this.source === "airtable" ? option.name : option;
        dropdownMenuItemTemplate.innerHTML = `<a class="filter-${optionText}" href="#${optionText}">${optionText}</a>`;
        dropdownMenu.appendChild(
          dropdownMenuItemTemplate.content.cloneNode(true)
        );
      });
    }
  
    async connectedCallback() {
      this.innerHTML = `<p class="loading">Getting information...</p>`;
      const shadowRoot = this.attachShadow({ mode: "open" });
      const template = document.createElement("template");
      template.innerHTML = `
        <div class="container">
          <div class="search" style="display:none">
            <section class="search no-code-list">
              <slot name="searched"><p class="searchmessage">Searching...</p></slot>
            </section>
          </div>
          <div class="rendered">
            <section class="no-code-list"><slot></slot></section>
          </div>
        </div>
      `;
      shadowRoot.appendChild(template.content.cloneNode(true));
  
      if (!document.querySelector(".content")?.dataset.appid) {
        return;
      }
  
      this.softrAppId = document.querySelector(".content").dataset.appid;
      this.origin = window.origin;
  
      await this.setVariant(this.type);
  
      const string = JSON.stringify(this.originalHTML);
  
      function findUniqueSubstrings(string, substringFormat) {
        const substrings = [];
        let startIndex = 0;
        while (startIndex < string.length) {
          const start = string.indexOf(substringFormat, startIndex);
          if (start === -1) {
            break;
          }
          let end = start + substringFormat.length;
          while (string[end] !== "}") {
            end++;
          }
          end++; // include the closing curly brace
          end++; // include the closing curly brace
          substrings.push(string.substring(start, end));
          startIndex = end;
        }
        return [...new Set(substrings)];
      }
  
      const placeholders = findUniqueSubstrings(string, "{{");
  
      const fieldNames = placeholders.map((placeholder) =>
        placeholder.replace("}}", "").replace("{{", "")
      );
  
      this.payload = {
        softrAppId: this.softrAppId,
        origin: this.origin,
        base: this.base,
        table: this.table,
        type: this.type,
        rowSize: this.rowSize,
        pageSize: this.pageSize,
        innerHTML: this.originalHTML,
        originalHTML: this.originalHTML,
        filter: this.filter,
        placeholders: placeholders,
        fieldNames: fieldNames,
        offset: "none",
        source: this.source,
        api: this.api,
      };
  
      this.getData(this.payload);
      if (this.source === "airtable" || !this.source) {
        this.getSearchData(this.payload);
        this.getFilterData(this.payload);
      }
    }
  }